/*
 * com/ngon/app/App.java
 * Victor Liu > mailto:victor@n-gon.com
 *
 * Copyright (C) 2004 Victor Liu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.ngon.app;

import com.ngon.gfx.ImageBuffer;
import com.ngon.gfx.ImageBufferFactory;
import java.awt.Graphics;
import java.awt.Image;
import java.io.InputStream;
import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URL;
import org.gaffer.tinyptc.TinyPTC;

/**
 * class App
 */
public abstract class App extends TinyPTC {
    
    protected boolean isRunningAsApplet;
    protected String appletSecurityEvader;
    
    public void init() {
        isRunningAsApplet = true;
    }
    
    public void kill() {
        if (! isRunningAsApplet) {
            System.exit(0);
        } else {
        }
    }
    
    /* ========================================================================
        IO OPS
       ========================================================================
     */
    public InputStream getDataStream(String file) {
        try {
            if (isRunningAsApplet) {
                URL u;
                if (file.startsWith("http://")) {
                    if (appletSecurityEvader != null) {
                        u = new URL(appletSecurityEvader + file);
                    } else {
                        u = new URL(file);
                    }
                } else {
                    u = new URL(getDocumentBase(), file);
                }
                return new BufferedInputStream(u.openStream());
            } else {
                return new DataInputStream(new FileInputStream(file));
            }
        } catch (NullPointerException npe) {
            // For some reason, this little catch fixes a one-time
            // java.lang.NullPointerException on Netscape 6.2 /
            // Mac OS 9.1 / MRJ 2.2.5 (no problem for IE on same system).
            // Does not happen if applet is re-run.
            System.out.println("Error opening stream -> retrying...");
            if (_thread == Thread.currentThread()) {
                return getDataStream(file);
            } else {
                return null;
            }
        } catch (IOException e) {
            System.err.println("Can't open " + file);
        }
        return null;
    }
    
    public Image getImage(String filename) throws IOException {
        InputStream in = getClass().getResourceAsStream(filename);
        if (in == null) {
            throw new IOException("Image " + filename + " not found");
        }
        final byte[] buffer = toByteArray(in);
        in.close();
        return getToolkit().createImage(buffer);
    }
    
    public ImageBuffer getImageBuffer(Image img) throws IOException {
        ImageBufferFactory ibFactory = new ImageBufferFactory(this);
        int w = img.getWidth(this);
        int h = img.getHeight(this);
        if (w <= 0 || h <= 0) {
            throw new IOException("Size of image is " + w + "x" + h);
        }
        ImageBuffer ib = ibFactory.create(w, h);
        Graphics g = ib.getGraphics();
        g.drawImage(img, 0, 0, this);
        ib.grabPixels();
        return ib;
    }
    
    // This method found on Java Developer forum
    static public byte[] toByteArray(final InputStream input) 
            throws IOException {
        int status = 0;
        final int blockSize = 5000;
        int totalBytesRead = 0;
        int blockCount = 1;
        byte[] dynamicBuffer = new byte[blockSize*blockCount];
        final byte[] buffer = new byte[blockSize];
        
        boolean endOfStream = false;
        while (!endOfStream) {
            int bytesRead = 0;
            if (input.available() != 0) {
                // data is waiting so read as
                // much as is available
                status = input.read(buffer);
                endOfStream = (status == -1);
                if (!endOfStream)
                    bytesRead = status;
            } else {
                // no data waiting so use the
                // one character read to block until
                // data is available or the end of the input stream is reached
                status = input.read();
                endOfStream = (status == -1);
                buffer[0] = (byte)status;
                if (!endOfStream) {
                    bytesRead = 1;
                }
            }
            
            if (!endOfStream) {
                if (totalBytesRead + bytesRead > blockSize*blockCount) {
                    // expand the size of the buffer
                    blockCount++;
                    final byte[] newBuffer = new byte[blockSize*blockCount];
                    System.arraycopy(dynamicBuffer, 0,
                                     newBuffer, 0, totalBytesRead);
                    dynamicBuffer = newBuffer;
                }
                System.arraycopy(buffer, 0,
                                 dynamicBuffer, totalBytesRead, bytesRead);
                totalBytesRead += bytesRead;
            }
        }
        
        // make a copy of the array of the exact length
        final byte[] result = new byte[totalBytesRead];
        if (totalBytesRead != 0) {
            System.arraycopy(dynamicBuffer, 0, result, 0, totalBytesRead);
        }
        
        return result;
    }
    
    public void print(String s) {
        if (! isRunningAsApplet) {
            System.out.print(s);
        } else {
        }
    }
    
    public void println(String s) {
        if (! isRunningAsApplet) {
            System.out.println(s);
        } else {
        }
    }
}
