/*
 * com/ngon/math/Gradient2D.java
 * Victor Liu > mailto:victor@n-gon.com
 *
 * Copyright (C) 2004 Victor Liu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.ngon.math;

import java.awt.Dimension;
import java.awt.Point;

/**
 * class Gradient2D
 */
final public class Gradient2D {
    
    public Point dVx = new Point();
    public Point dVy = new Point();
    
    static public Gradient2D[] gradient2D(Point[] field, Dimension d) {
        Gradient2D[] grad2D = new Gradient2D[d.width * d.height];
        int p = 0;
        for (int j = 0; j < d.height; j++) {
            for (int i = 0; i < d.width; i++, p++) {
                grad2D[p] = new Gradient2D();
                Point v0 = field[p];
                Point v;
                int n, dVx, dVy;
                
                n = 0;
                dVx = 0;
                dVy = 0;
                if (i > 0) {
                    v = field[p-1];
                    dVx += v0.x - v.x;
                    dVy += v0.y - v.y;
                    n++;
                }
                if (i < d.width-1) {
                    v = field[p+1];
                    dVx += v.x - v0.x;
                    dVy += v.y - v0.y;
                    n++;
                }
                if (n > 1) {
                    dVx /= n;
                    dVy /= n;
                }
                grad2D[p].dVx.x = dVx;
                grad2D[p].dVy.x = dVy;
                
                n = 0;
                dVx = 0;
                dVy = 0;
                if (j > 0) {
                    v = field[p-d.width];
                    dVx += v0.x - v.x;
                    dVy += v0.y - v.y;
                    n++;
                }
                if (j < d.height-1) {
                    v = field[p+d.width];
                    dVx += v.x - v0.x;
                    dVy += v.y - v0.y;
                    n++;
                }
                if (n > 1) {
                    dVx /= n;
                    dVy /= n;
                }
                grad2D[p].dVx.y = dVx;
                grad2D[p].dVy.y = dVy;
            }
        }
        return grad2D;
    }
    
    static public int div(Gradient2D grad) {
        return grad.dVx.x + grad.dVy.y;
    }
    
    static public int curl(Gradient2D grad) {
        return grad.dVy.x - grad.dVx.y;
    }
}

